"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.LocalFileUploader = void 0;
const prop_types_1 = __importDefault(require("prop-types"));
const react_1 = __importStar(require("react"));
const components_datatable_legacy_1 = require("@react_db_client/components.datatable-legacy");
const constants_client_types_1 = require("@react_db_client/constants.client-types");
const react_csv_reader_1 = __importDefault(require("react-csv-reader"));
const HeadingsMapper_1 = __importDefault(require("./HeadingsMapper"));
const HeadingEditor = ({ headings: headingsIn, acceptHeadings }) => {
    const [headings, setHeadings] = (0, react_1.useState)(headingsIn);
    const updateHeadings = (i, t) => {
        const headingsEdited = [...headings];
        headingsEdited[i] = t;
        setHeadings(headingsEdited);
    };
    return (react_1.default.createElement("form", { onSubmit: (e) => {
            e.preventDefault();
            acceptHeadings(headings);
        }, className: "localFileUploader_headingForm" },
        headings &&
            headings.map((h, i) => (react_1.default.createElement("input", { type: "text", value: h, onChange: (e) => updateHeadings(i, e.target.value) }))),
        react_1.default.createElement("button", { type: "submit", className: "button-one" }, "Update Headings")));
};
HeadingEditor.propTypes = {
    headings: prop_types_1.default.arrayOf(prop_types_1.default.string).isRequired,
    acceptHeadings: prop_types_1.default.func.isRequired,
};
const getHeadingsFromRawData = (data, headings) => {
    const columnCount = Math.max(...data.map((row) => row.length));
    const headingsNew = Array.from({ length: columnCount }, (_, i) => headings ? headings[i] || i : i);
    return headingsNew;
};
/**
 * A local file uploader that allows loading csv files and assigning headers.
 *
 */
const LocalFileUploader = ({ label, onAccept, onChange, mapToHeadings, showAcceptButton, }) => {
    const [hasHeader, setHasHeader] = (0, react_1.useState)(false);
    const [skipRows, setSkipRows] = (0, react_1.useState)(0);
    const [rawData, setRawData] = (0, react_1.useState)();
    const [headingsCustom, setHeadingsCustom] = (0, react_1.useState)();
    // eslint-disable-next-line no-unused-vars
    const [, setFileInfo] = (0, react_1.useState)(null);
    const [error, setError] = (0, react_1.useState)();
    const dataCropped = (0, react_1.useMemo)(() => {
        return rawData ? rawData.slice(skipRows) : [];
    }, [rawData, skipRows]);
    const dataSliced = (0, react_1.useMemo)(() => {
        return hasHeader ? dataCropped.slice(1) : dataCropped;
    }, [hasHeader, dataCropped]);
    const rawHeadings = (0, react_1.useMemo)(() => {
        const headingsIn = hasHeader ? dataCropped[0] : null;
        return dataCropped ? getHeadingsFromRawData(dataCropped, headingsIn) : [];
    }, [dataCropped, hasHeader]);
    const headings = (0, react_1.useMemo)(() => {
        const headingsIn = headingsCustom || rawHeadings;
        return dataCropped ? getHeadingsFromRawData(dataCropped, headingsIn) : [];
    }, [dataCropped, headingsCustom, rawHeadings]);
    const dataDict = (0, react_1.useMemo)(() => {
        const dataUpdated = dataSliced.map((row, i) => row.reduce((d, c, j) => {
            const nd = Object.assign(Object.assign({}, d), { uid: i });
            const heading = j < headings.length ? headings[j] : j;
            if (c !== null)
                nd[heading] = c;
            return nd;
        }, {}));
        if (onChange)
            onChange(dataUpdated);
        return dataUpdated;
    }, [dataSliced, headings, onChange]);
    const setHeadings = setHeadingsCustom;
    const handleHasHeaderCheckChange = () => {
        setHasHeader((prev) => !prev);
        setHeadingsCustom(null);
    };
    const handleUpdateSkipRowCount = (val) => {
        setSkipRows(val);
        setHeadingsCustom(null);
    };
    const handleFileUploaded = (dataNew, fileInfoNew) => {
        setRawData(dataNew);
        setFileInfo(fileInfoNew);
        setError(null);
        setHeadingsCustom(null);
    };
    const parserOptions = {
        header: false,
        dynamicTyping: true,
        skipEmptyLines: false,
        transformHeader: (h) => h.toLowerCase().replace(/\W/g, '_'),
    };
    return (react_1.default.createElement("div", { className: "localFileUploader sectionWrapper" },
        react_1.default.createElement("section", { className: "localFileUploader_selectFileSection" },
            react_1.default.createElement("h2", null, "1. Select File"),
            react_1.default.createElement("div", null,
                react_1.default.createElement("label", { htmlFor: "lfu-fileHasHeaderCheckbox" }, "File has a header?"),
                react_1.default.createElement("input", { id: "lfu-fileHasHeaderCheckbox", className: "headerCheckbox", type: "checkbox", onChange: () => handleHasHeaderCheckChange(), checked: hasHeader || false })),
            react_1.default.createElement("div", null,
                react_1.default.createElement("label", null, "Skip Rows: "),
                react_1.default.createElement("input", { type: "number", name: "skipRowsInput", className: "skipRowsInput", onChange: (e) => handleUpdateSkipRowCount(e.target.value), value: skipRows })),
            react_1.default.createElement(react_csv_reader_1.default, { label: label, parserOptions: parserOptions, onError: setError, onFileLoaded: handleFileUploaded }),
            react_1.default.createElement("div", null, error && JSON.stringify(error))),
        rawHeadings && rawHeadings.length > 0 && !mapToHeadings && (react_1.default.createElement("section", { className: "localFileUploader_headingEditor" },
            react_1.default.createElement("h2", null, "Edit Headings"),
            react_1.default.createElement(HeadingEditor, { key: rawHeadings.join(','), headings: rawHeadings, acceptHeadings: setHeadings }))),
        rawHeadings && rawHeadings.length > 0 && mapToHeadings && (react_1.default.createElement("section", { className: "localFileUploader_headingMapper" },
            react_1.default.createElement("h2", null, "Map Headings"),
            react_1.default.createElement(HeadingsMapper_1.default, { key: rawHeadings.join(','), headings: rawHeadings, mapToHeadings: mapToHeadings, handleAccept: setHeadings, errorCallback: setError }))),
        headings && dataDict && headings.length > 0 && (react_1.default.createElement("section", { className: "localFileUploader_previewDataSection" },
            react_1.default.createElement("h2", null, "Preview Data"),
            react_1.default.createElement(components_datatable_legacy_1.DataTableSimple, { key: headings.join(','), id: "previewDataTable", headingsData: headings.map((h) => {
                    var _a;
                    return ({
                        uid: h,
                        label: String(((_a = mapToHeadings === null || mapToHeadings === void 0 ? void 0 : mapToHeadings.find((mh) => mh.uid === h)) === null || _a === void 0 ? void 0 : _a.label) || h),
                        type: constants_client_types_1.EFilterType.text,
                    });
                }), tableData: dataDict, tableHeight: 300, config: {}, showTotals: false }))),
        showAcceptButton && (react_1.default.createElement("section", { className: "localFileUploader_acceptButtonsSection" },
            react_1.default.createElement("button", { type: "button", className: "button-two", disabled: !rawData, onClick: () => onAccept(dataDict) }, "Accept File")))));
};
exports.LocalFileUploader = LocalFileUploader;
exports.LocalFileUploader.propTypes = {
    label: prop_types_1.default.string.isRequired,
    onAccept: prop_types_1.default.func.isRequired,
    onChange: prop_types_1.default.func,
    //  @ts-ignore
    mapToHeadings: prop_types_1.default.arrayOf(prop_types_1.default.shape({
        uid: prop_types_1.default.string.isRequired,
        label: prop_types_1.default.string,
    })),
    showAcceptButton: prop_types_1.default.bool,
};
exports.LocalFileUploader.defaultProps = {
    onChange: () => { },
    mapToHeadings: [],
    showAcceptButton: false,
};
exports.default = exports.LocalFileUploader;
//# sourceMappingURL=local-file-uploader.js.map